@@ -33,6 +33,10 @@ namespace Tomboy
 		Gtk.Button disable_addin_button;
 		Gtk.Button addin_prefs_button;
 		Gtk.Button addin_info_button;
+		
+		private Gtk.RadioButton promptOnConflictRadio;
+		private Gtk.RadioButton renameOnConflictRadio;
+		private Gtk.RadioButton overwriteOnConflictRadio;
 
 		/// <summary>
 		/// Keep track of the opened addin prefs dialogs so other windows
@@ -85,6 +89,8 @@ namespace Tomboy
 			    new Gtk.Label (Catalog.GetString ("Add-ins")));
 
 			VBox.PackStart (notebook, true, true, 0);
+			
+			addin_manager.ApplicationAddinListChanged += OnAppAddinListChanged;
 
 
 			// Ok button...
@@ -348,6 +354,7 @@ namespace Tomboy
 			syncAddinStore = new Gtk.ListStore (typeof (SyncServiceAddin));
 			syncAddinIters = new Dictionary<string,Gtk.TreeIter> ();
 			SyncServiceAddin [] addins = Tomboy.DefaultNoteManager.AddinManager.GetSyncServiceAddins ();
+			Array.Sort (addins, CompareSyncAddinsByName);
 			foreach (SyncServiceAddin addin in addins) {
 				Gtk.TreeIter iter = syncAddinStore.Append ();
 				syncAddinStore.SetValue (iter, 0, addin);
@@ -395,6 +402,9 @@ namespace Tomboy
 				l.Yalign = 0.5f;
 				syncAddinPrefsWidget = l;
 			}
+			if (syncAddinPrefsWidget != null && addin_id != null &&
+			    syncAddinIters.ContainsKey (addin_id) && selectedSyncAddin.IsConfigured)
+				syncAddinPrefsWidget.Sensitive = false;
 			
 			syncAddinPrefsWidget.Show ();
 			syncAddinPrefsContainer = new Gtk.VBox (false, 0);
@@ -406,25 +416,36 @@ namespace Tomboy
 			bbox.Spacing = 4;
 			bbox.LayoutStyle = Gtk.ButtonBoxStyle.End;
 			
+			// "Advanced..." button to bring up extra sync config dialog
+			Gtk.Button advancedConfigButton = new Gtk.Button (Catalog.GetString ("Advanced..."));
+			advancedConfigButton.Clicked += OnAdvancedSyncConfigButton;
+			advancedConfigButton.Show ();
+			bbox.PackStart (advancedConfigButton, false, false, 0);
+			bbox.SetChildSecondary (advancedConfigButton, true);
+			
 			resetSyncAddinButton = new Gtk.Button (Gtk.Stock.Clear);
 			resetSyncAddinButton.Clicked += OnResetSyncAddinButton;
+			resetSyncAddinButton.Sensitive =
+				(selectedSyncAddin != null &&
+				 addin_id == selectedSyncAddin.Id &&
+				 selectedSyncAddin.IsConfigured);
 			resetSyncAddinButton.Show ();
 			bbox.PackStart (resetSyncAddinButton, false, false, 0);
 			
+			// TODO: Tabbing should go directly from sync prefs widget to here
+			// TODO: Consider connecting to "Enter" pressed in sync prefs widget
 			saveSyncAddinButton = new Gtk.Button (Gtk.Stock.Save);
 			saveSyncAddinButton.Clicked += OnSaveSyncAddinButton;
+			saveSyncAddinButton.Sensitive =
+				(selectedSyncAddin != null &&
+				 (addin_id != selectedSyncAddin.Id || !selectedSyncAddin.IsConfigured));
 			saveSyncAddinButton.Show ();
 			bbox.PackStart (saveSyncAddinButton, false, false, 0);
 			
-			if (selectedSyncAddin != null && selectedSyncAddin.IsConfigured) {
-				saveSyncAddinButton.Sensitive = false;
-				resetSyncAddinButton.Sensitive = true;
-				syncAddinCombo.Sensitive = false;
-			} else {
-				saveSyncAddinButton.Sensitive = true;
-				resetSyncAddinButton.Sensitive = false;
-				syncAddinCombo.Sensitive = true;
-			}
+			syncAddinCombo.Sensitive =
+				(selectedSyncAddin == null ||
+				 addin_id != selectedSyncAddin.Id ||
+				 !selectedSyncAddin.IsConfigured);
 			
 			bbox.Show ();
 			vbox.PackStart (bbox, false, false, 0);
@@ -432,6 +453,11 @@ namespace Tomboy
 			vbox.ShowAll ();
 			return vbox;
 		}
+			
+		private int CompareSyncAddinsByName (SyncServiceAddin addin1, SyncServiceAddin addin2)
+		{
+			return addin1.Name.CompareTo (addin2.Name);
+		}
 		
 		private void ComboBoxTextDataFunc (Gtk.CellLayout cell_layout, Gtk.CellRenderer cell,
 			Gtk.TreeModel tree_model, Gtk.TreeIter iter)
@@ -807,6 +833,131 @@ namespace Tomboy
 							  desc.ToString ());
 		}
 		
+		private void OnAdvancedSyncConfigButton (object sender, EventArgs args)
+		{
+			// Get saved behavior
+			SyncTitleConflictResolution savedBehavior = SyncTitleConflictResolution.Cancel;
+			object dlgBehaviorPref = Preferences.Get (Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR);
+			if (dlgBehaviorPref != null && dlgBehaviorPref is int) // TODO: Check range of this int
+				savedBehavior = (SyncTitleConflictResolution)dlgBehaviorPref;
+			
+			// Create dialog
+			Gtk.Dialog advancedDlg =
+				new Gtk.Dialog (Catalog.GetString ("Other Synchronization Options"),
+				                this,
+				                Gtk.DialogFlags.DestroyWithParent | Gtk.DialogFlags.Modal,
+				                Gtk.Stock.Close, Gtk.ResponseType.Close);
+			
+			// Populate dialog
+			Gtk.Label label =
+				new Gtk.Label (Catalog.GetString ("When a conflict is detected between " +
+				                                  "a local note and a note on the configured " +
+				                                  "synchronization server:"));
+			label.Wrap = true;
+			label.Xalign = 0;
+			
+			promptOnConflictRadio =
+				new Gtk.RadioButton (Catalog.GetString ("Always ask me what to do."));
+			promptOnConflictRadio.Toggled += OnConflictOptionToggle;
+			
+			renameOnConflictRadio =
+				new Gtk.RadioButton (promptOnConflictRadio, Catalog.GetString ("Rename my local note."));
+			renameOnConflictRadio.Toggled += OnConflictOptionToggle;
+			
+			overwriteOnConflictRadio =
+				new Gtk.RadioButton (promptOnConflictRadio, Catalog.GetString ("Replace my local note with the server's update."));
+			overwriteOnConflictRadio.Toggled += OnConflictOptionToggle;
+			
+			switch (savedBehavior) {
+			case SyncTitleConflictResolution.RenameExistingNoUpdate:
+				renameOnConflictRadio.Active = true;
+				break;
+			case SyncTitleConflictResolution.OverwriteExisting:
+				overwriteOnConflictRadio.Active = true;
+				break;
+			default:
+				promptOnConflictRadio.Active = true;
+				break;
+			}
+			
+			Gtk.VBox vbox = new Gtk.VBox ();
+			vbox.BorderWidth = 18;
+			
+			vbox.PackStart (promptOnConflictRadio);
+			vbox.PackStart (renameOnConflictRadio);
+			vbox.PackStart (overwriteOnConflictRadio);
+			
+			advancedDlg.VBox.PackStart (label, false, false, 6);
+			advancedDlg.VBox.PackStart (vbox, false, false, 0);
+			
+			advancedDlg.ShowAll ();
+			
+			// Run dialog
+			advancedDlg.Run ();
+			advancedDlg.Destroy ();
+		}
+		
+		private void OnConflictOptionToggle (object sender, EventArgs args)
+		{
+			SyncTitleConflictResolution newBehavior = SyncTitleConflictResolution.Cancel;
+			
+			if (renameOnConflictRadio.Active)
+				newBehavior = SyncTitleConflictResolution.RenameExistingNoUpdate;
+			else if (overwriteOnConflictRadio.Active)
+				newBehavior = SyncTitleConflictResolution.OverwriteExisting;
+			
+			Preferences.Set (Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR,
+			                 (int) newBehavior);
+		}
+		
+		private void OnAppAddinListChanged (object sender, EventArgs args)
+		{
+			SyncServiceAddin [] newAddinsArray = Tomboy.DefaultNoteManager.AddinManager.GetSyncServiceAddins ();
+			Array.Sort (newAddinsArray, CompareSyncAddinsByName);
+			List<SyncServiceAddin> newAddins = new List<SyncServiceAddin> (newAddinsArray);
+			
+			// Build easier-to-navigate list if addins currently in the combo
+			List<SyncServiceAddin> currentAddins = new List<SyncServiceAddin> ();
+			foreach (object [] currentRow in syncAddinStore) {
+				SyncServiceAddin currentAddin = currentRow [0] as SyncServiceAddin;
+				if (currentAddin != null)
+					currentAddins.Add (currentAddin);
+			}
+			
+			// Add new addins
+			// TODO: Would be nice to insert these alphabetically instead
+			foreach (SyncServiceAddin newAddin in newAddins) {
+				if (!currentAddins.Contains (newAddin)) {
+					Gtk.TreeIter iter = syncAddinStore.Append ();
+					syncAddinStore.SetValue (iter, 0, newAddin);
+					syncAddinIters [newAddin.Id] = iter;
+				}
+			}
+			
+			// Remove deleted addins
+			foreach (SyncServiceAddin currentAddin in currentAddins) {
+				if (!newAddins.Contains (currentAddin)) {
+					Gtk.TreeIter iter = syncAddinIters [currentAddin.Id];
+					syncAddinStore.Remove (ref iter);
+					syncAddinIters.Remove (currentAddin.Id);
+
+					// FIXME: Lots of hacky stuff in here...rushing before freeze
+					if (currentAddin == selectedSyncAddin) {
+						if (syncAddinPrefsWidget != null &&
+						    !syncAddinPrefsWidget.Sensitive)
+							OnResetSyncAddinButton (null, null);
+						
+						Gtk.TreeIter active_iter;
+						if (syncAddinStore.GetIterFirst (out active_iter) == true) {
+							syncAddinCombo.SetActiveIter (active_iter);
+						} else {
+							OnSyncAddinComboChanged (null, null);
+						}
+					}
+				}
+			}
+		}
+		
 		void OnSyncAddinComboChanged (object sender, EventArgs args)
 		{
 			if (syncAddinPrefsWidget != null) {
@@ -832,8 +983,16 @@ namespace Tomboy
 			
 					syncAddinPrefsWidget.Show ();
 					syncAddinPrefsContainer.PackStart (syncAddinPrefsWidget, true, true, 0);
+					
+					resetSyncAddinButton.Sensitive = false;
+					saveSyncAddinButton.Sensitive = true;
 				}
+			} else {
+				selectedSyncAddin = null;
+				resetSyncAddinButton.Sensitive = false;
+				saveSyncAddinButton.Sensitive = false;
 			}
+				
 		}
 		
 		void OnResetSyncAddinButton (object sender, EventArgs args)
@@ -841,24 +1000,41 @@ namespace Tomboy
 			if (selectedSyncAddin == null)
 				return;
 			
-			// Prompt the user about what they're about to do since
-			// it's not really recommended to switch back and forth
-			// between sync services.
-			// Prompt the user first about enabling fuse
-			HIGMessageDialog dialog = 
-				new HIGMessageDialog (null,
-						      Gtk.DialogFlags.Modal,
-						      Gtk.MessageType.Question,
-						      Gtk.ButtonsType.YesNo,
-						      Catalog.GetString ("WARNING: Are you sure?"),
-						      Catalog.GetString (
-					"Clearing your synchronization settings is not recommended.  " +
-					"You may be forced to synchronize all of your notes again " +
-					"when you save new settings."));
-			int response = dialog.Run ();
-			dialog.Destroy ();
-			if (response == (int) Gtk.ResponseType.No)
-				return;
+			// User doesn't get a choice if this is invoked by disabling the addin
+			// FIXME: null sender check is lame!
+			if (sender != null) {
+				// Prompt the user about what they're about to do since
+				// it's not really recommended to switch back and forth
+				// between sync services.
+				HIGMessageDialog dialog = 
+					new HIGMessageDialog (null,
+							      Gtk.DialogFlags.Modal,
+							      Gtk.MessageType.Question,
+							      Gtk.ButtonsType.YesNo,
+							      Catalog.GetString ("WARNING: Are you sure?"),
+							      Catalog.GetString (
+						"Clearing your synchronization settings is not recommended.  " +
+						"You may be forced to synchronize all of your notes again " +
+						"when you save new settings."));
+				int response = dialog.Run ();
+				dialog.Destroy ();
+				if (response == (int) Gtk.ResponseType.No)
+					return;
+			} else { // FIXME: Weird place for this to go.  User should be able to cancel disabling of addin, anyway
+				HIGMessageDialog dialog = 
+					new HIGMessageDialog (null,
+							      Gtk.DialogFlags.Modal,
+							      Gtk.MessageType.Info,
+							      Gtk.ButtonsType.Ok,
+							      Catalog.GetString ("Resetting Synchronization Settings"),
+							      Catalog.GetString (
+					                                         "You have disabled the configured synchronization service.  " +
+					                                         "Your synchronization settings will now be cleared.  " +
+					                                         "You may be forced to synchronize all of your notes again " +
+					                                         "when you save new settings"));
+				dialog.Run ();
+				dialog.Destroy ();
+			}
 			
 			try {
 				selectedSyncAddin.ResetConfiguration ();
@@ -871,6 +1047,25 @@ namespace Tomboy
 				Preferences.SYNC_SELECTED_SERVICE_ADDIN,
 				String.Empty);
 			
+			// Reset conflict handling behavior
+			Preferences.Set (
+			                 Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR,
+			                 Preferences.GetDefault (Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR));
+			
+			NukeSyncClientManifest ();
+				
+			syncAddinCombo.Sensitive = true;
+			resetSyncAddinButton.Sensitive = false;
+			saveSyncAddinButton.Sensitive = true;
+			if (syncAddinPrefsWidget != null)
+				syncAddinPrefsWidget.Sensitive = true;
+		}
+		
+		// TODO: This class shouldn't know about the manifest.
+		//       Abstract this out somehow (ie, SyncManager.ResetClient()
+		//       or something like that).
+		void NukeSyncClientManifest ()
+		{
 			// Nuke ~/.tomboy/manifest.xml
 			string clientManifestPath = System.IO.Path.Combine ( 
 					Tomboy.DefaultNoteManager.NoteDirectoryPath,
@@ -884,11 +1079,6 @@ namespace Tomboy
 						e.Message);
 				}
 			}
-				
-			syncAddinCombo.Sensitive = true;
-			syncAddinPrefsWidget.Sensitive = true;
-			resetSyncAddinButton.Sensitive = false;
-			saveSyncAddinButton.Sensitive = true;
 		}
 		
 		/// <summary>
@@ -900,11 +1090,19 @@ namespace Tomboy
 				return;
 			
 			bool saved = false;
+			string errorMsg = null;
 			try {
+				GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
+				GdkWindow.Display.Flush ();
 				saved = selectedSyncAddin.SaveConfiguration ();
+			} catch (TomboySyncException syncEx) {
+				errorMsg = syncEx.Message;
 			} catch (Exception e) {
-				Logger.Debug ("Error calling {0}.SaveConfiguration: {1}\n{2}",
+				Logger.Debug ("Unexpected error calling {0}.SaveConfiguration: {1}\n{2}",
 					selectedSyncAddin.Id, e.Message, e.StackTrace);
+			} finally {
+				GdkWindow.Cursor = null;
+				GdkWindow.Display.Flush ();
 			}
 			
 			HIGMessageDialog dialog; 
@@ -917,6 +1115,8 @@ namespace Tomboy
 				syncAddinPrefsWidget.Sensitive = false;
 				resetSyncAddinButton.Sensitive = true;
 				saveSyncAddinButton.Sensitive = false;
+			
+				NukeSyncClientManifest ();
 
 				// Give the user a visual letting them know that connecting
 				// was successful.
@@ -945,14 +1145,18 @@ namespace Tomboy
 
 				// Give the user a visual letting them know that connecting
 				// was successful.
+				if (errorMsg == null)
+					errorMsg = Catalog.GetString ("Sorry, but something went wrong.  " +
+					                              "Please check your information and " +
+					                              "try again.  The ~/.tomboy.log might " +
+					                              "be useful too.");
 				dialog = 
 					new HIGMessageDialog (this,
 							      Gtk.DialogFlags.Modal,
 							      Gtk.MessageType.Warning,
 							      Gtk.ButtonsType.Close,
 							      Catalog.GetString ("Error connecting :("),
-							      Catalog.GetString (
-						"Sorry, but something went wrong.  Please check your information and try again.  The ~/.tomboy.log might be useful too."));
+							      errorMsg);
 				dialog.Run ();
 				dialog.Destroy ();
 			}
